defmodule Prymn.Application do
  # See https://hexdocs.pm/elixir/Application.html
  # for more information on OTP Applications
  @moduledoc false

  use Application

  @impl true
  def start(_type, _args) do
    children = [
      PrymnWeb.Telemetry,
      Prymn.Repo,
      {DNSCluster, query: Application.get_env(:prymn, :dns_cluster_query) || :ignore},
      {Phoenix.PubSub, name: Prymn.PubSub},
      {Finch, name: Prymn.Finch},
      {Oban, Application.fetch_env!(:prymn, Oban)},
      Prymn.Agents.Store,
      Prymn.Messaging.ConnectionSupervisor,
      {Task.Supervisor, name: Prymn.TaskSupervisor},
      PrymnWeb.Endpoint
    ]

    # See https://hexdocs.pm/elixir/Supervisor.html
    # for other strategies and supported options
    opts = [strategy: :one_for_one, name: Prymn.Supervisor]
    Supervisor.start_link(children, opts)
  end

  # Tell Phoenix to update the endpoint configuration
  # whenever the application is updated.
  @impl true
  def config_change(changed, _new, removed) do
    PrymnWeb.Endpoint.config_change(changed, removed)
    :ok
  end
end
